#
#  converts a tab-delimited ASCII file (exported from Excel) 
#  into Western Geophysical velocity format.  
#  The original Excel file MUST be in a certain format to begin with:
#         (column number 1 = barcode
#          column number 2 = department line name
#          column number 3 = shot point
#          column number 4 = CDP
#          column number 5 = Two way time (ms)
#          column number 6 = Root Mean Square velocity
#          column number 7 = Interval Velocity
#          column number 8 = FT or metres (i.e. FT = feet)
#  Parameters: column number to use as the first column (usually 0)
#  Input:  STDIN  (ASCII tab-delimited)
#  Output: STDOUT (ASCII Western vel format)
#  Example: perl conv2western.pl 0 < input.txt > output.txt
#
#  Assumes that a different shots point delineates a new velocity function.
#

$firstcol = shift ;
while (<>) { 
  @asciilin = split /\t/ ; 
  $ba = $asciilin[$firstcol + 0] ;    # asset item barcode
  $li = $asciilin[$firstcol + 1] ;    # seismic line name
  $sh = $asciilin[$firstcol + 2] ;    # shot number
  $cd = $asciilin[$firstcol + 3] ;    # cdp  number
  $ti = $asciilin[$firstcol + 4] ;    # time
  $ve = $asciilin[$firstcol + 5] ;    # stack vel
  $vi = $asciilin[$firstcol + 6] ;    # interval vel
  $ft = $asciilin[$firstcol + 7] ;    # "FT" or metres
  $barcode = &newvalset($ba, $barcode) ;
  $line    = &newvalset($li, $line) ;
  $shot    = &newvalset($sh, $shot    ) ;
  $cdp     = &newvalset($cd, $cdp) ;
  $time    = &newvalset($ti, "") ;
  $vel     = &newvalset($ve, "") ;
  $vint    = &newvalset($vi, $vint) ;
  $ft      = &newvalset($ft, "") ;    # blank FT means metres!
  $newset  = 0 ;
  if ( $lastline ne $line ) { 
    print "\nLINE      ", $line  ;
    }
  if (($sh ne "") || ( $cd ne "")) { 
    $newset = 1; 
    }
  if ( $newset ) { 
    $shot =~ s/[ABCD]$// ;                 # eliminate alpha suffixes to shot point numbering
    printf( "\nSPNT %20s\n", $shot )  ;    # print SP
    $ntimes = 0 ;
    }
  if ( $time ne "" ) { 
    if ( $ntimes > 0 ) { 
      if (( $time =~ m/\./  ) || ( $ntimes > 3 )) {      # is time in seconds?
        if ( $time < 10.0 ) { 
          $time = $time * 1000.0 ;  # if time in seconds, convert to milliseconds
          }
        }
      }
    }
  if ( $vel ne "" ) { 
    if ( $ft eq "FT" ) {       # is velocity in ft/s ?
      $vel = $vel * 0.3048 ;   # convert to m/s from FT
      }
    }
  if ( $time ne "" ) { 
    if ( ( $ntimes % 5 ) == 0 ) {
      if ( $ntimes != 0 ) { 
        print "\n" ;
        }
      print("VELF                ") ;
      }
    printf("%5d%5d",$time, $vel) ;
    $ntimes++ ;
    }
  $lastline = $line ;
  }
sub newvalset($$) 
{
  my $newv = shift ;
  my $oldv = shift ;
  my $returnv = $oldv ;
  if ( $newv ne "") {
    if ( $newv ne $oldv ) {
      $returnv = $newv ; 
      }
    }
  return ($returnv); 
}
